from django.contrib import admin
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _


class CountryFilter(admin.FieldListFilter):
    """
    A country filter for Django admin that only returns a list of countries
    related to the model.
    """
    title = _('Country')

    def expected_parameters(self):
        return [self.field.name]

    def choices(self, changelist):
        value = self.used_parameters.get(self.field.name)
        yield {
            'selected': value is None,
            'query_string': changelist.get_query_string(
                {}, [self.field.name]),
            'display': _('All'),
        }
        for lookup, title in self.lookup_choices(changelist):
            yield {
                'selected': value == force_text(lookup),
                'query_string': changelist.get_query_string(
                    {self.field.name: lookup}, []),
                'display': title,
            }

    def lookup_choices(self, changelist):
        qs = changelist.model._default_manager.all()
        codes = set(
            qs.distinct()
            .order_by(self.field.name)
            .values_list(self.field.name, flat=True))
        for k, v in self.field.get_choices(include_blank=False):
            if k in codes:
                yield k, v



from . import fields, serializer_fields
from rest_framework import serializers


class CountryFieldMixin(object):

    def build_standard_field(self, field_name, model_field):
        field_class, field_kwargs = (
            super(CountryFieldMixin, self)
            .build_standard_field(field_name, model_field)
        )
        if (
            # Only deal with CountryFields.
            not isinstance(model_field, fields.CountryField) or
            # Some other mixin has changed the field class already!
            field_class is not serializers.ChoiceField
        ):
            return field_class, field_kwargs
        field_kwargs['countries'] = model_field.countries
        del field_kwargs['choices']
        if not model_field.multiple:
            field_class = serializer_fields.CountryField
        else:
            field_class = serializers.ListField
            child_field = serializer_fields.CountryField(**field_kwargs)
            field_kwargs = {'child': child_field}
            if 'max_length' in serializers.ListField.default_error_messages:
                # Added in DRF 3.5.4
                field_kwargs['max_length'] = len(child_field.countries)
        return field_class, field_kwargs



"""
This file provides zest.releaser entrypoints using when releasing new
django-countries versions.
"""
import os

from txclib.commands import cmd_pull
from txclib.utils import find_dot_tx
from txclib.log import logger
from zest.releaser.utils import ask, execute_command
from django.core.management import call_command
import django_countries


def translations(data):
    if data['name'] != 'django-countries':
        return

    if not ask('Pull translations from transifex and compile', default=True):
        return

    _handlers = logger.handlers
    logger.handlers = []
    try:
        cmd_pull(argv=['-a', '--minimum-perc=60'], path_to_tx=find_dot_tx())
    finally:
        logger.handlers = _handlers

    _cwd = os.getcwd()
    os.chdir(os.path.dirname(django_countries.__file__))
    try:
        call_command('compilemessages')
        execute_command(['git', 'add', 'locale'])
    finally:
        os.chdir(_cwd)



from __future__ import unicode_literals

try:
    from urllib import parse as urlparse
except ImportError:
    import urlparse   # Python 2
try:
    basestring
except NameError:
    basestring = str   # Python 3

from django import forms
from django.core import checks, exceptions
from django.contrib.admin.filters import FieldListFilter
from django.db.models.fields import CharField, BLANK_CHOICE_DASH
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.html import escape as escape_html
from django.utils.functional import lazy

from django_countries import countries, ioc_data, widgets, filters
from django_countries.conf import settings


def country_to_text(value):
    if hasattr(value, 'code'):
        value = value.code
    if value is None:
        return None
    return force_text(value)


class TemporaryEscape(object):
    __slots__ = ['country', 'original_escape']

    def __init__(self, country):
        self.country = country

    def __bool__(self):
        return self.country._escape

    __nonzero__ = __bool__

    def __enter__(self):
        self.original_escape = self.country._escape
        self.country._escape = True

    def __exit__(self, type, value, traceback):
        self.country._escape = self.original_escape


@python_2_unicode_compatible
class Country(object):

    def __init__(
            self, code, flag_url=None, str_attr='code', custom_countries=None):
        self.flag_url = flag_url
        self._escape = False
        self._str_attr = str_attr
        if custom_countries is countries:
            custom_countries = None
        self.custom_countries = custom_countries
        # Attempt to convert the code to the alpha2 equivalent, but this
        # is not meant to be full validation so use the given code if no
        # match was found.
        self.code = self.countries.alpha2(code) or code

    def __str__(self):
        return force_text(getattr(self, self._str_attr) or '')

    def __eq__(self, other):
        return force_text(self.code or '') == force_text(other or '')

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash(force_text(self))

    def __repr__(self):
        args = ['code={country.code!r}']
        if self.flag_url is not None:
            args.append('flag_url={country.flag_url!r}')
        if self._str_attr != 'code':
            args.append('str_attr={country._str_attr!r}')
        args = ', '.join(args).format(country=self)
        return '{name}({args})'.format(name=self.__class__.__name__, args=args)

    def __bool__(self):
        return bool(self.code)

    __nonzero__ = __bool__   # Python 2 compatibility.

    def __len__(self):
        return len(force_text(self))

    @property
    def countries(self):
        return self.custom_countries or countries

    @property
    def escape(self):
        return TemporaryEscape(self)

    def maybe_escape(self, text):
        if not self.escape:
            return text
        return escape_html(text)

    @property
    def name(self):
        return self.maybe_escape(self.countries.name(self.code))

    @property
    def alpha3(self):
        return self.countries.alpha3(self.code)

    @property
    def numeric(self):
        return self.countries.numeric(self.code)

    @property
    def numeric_padded(self):
        return self.countries.numeric(self.code, padded=True)

    @property
    def flag(self):
        if not self.code:
            return ''
        flag_url = self.flag_url
        if flag_url is None:
            flag_url = settings.COUNTRIES_FLAG_URL
        url = flag_url.format(
            code_upper=self.code, code=self.code.lower())
        if not url:
            return ''
        url = urlparse.urljoin(settings.STATIC_URL, url)
        return self.maybe_escape(url)

    @property
    def flag_css(self):
        """
        Output the css classes needed to display an HTML element as a flag
        sprite.

        Requires the use of 'flags/sprite.css' or 'flags/sprite-hq.css'.
        Usage example::

            <i class="{{ ctry.flag_css }}" aria-label="{{ ctry.code }}></i>
        """
        if not self.code:
            return ''
        return 'flag-sprite flag-{} flag-_{}'.format(*self.code.lower())

    @property
    def unicode_flag(self):
        """
        Generate a unicode flag for the given country.

        The logic for how these are determined can be found at:

        https://en.wikipedia.org/wiki/Regional_Indicator_Symbol

        Currently, these glyphs appear to only be supported on OS X and iOS.
        """
        if not self.code:
            return ''

        # Don't really like magic numbers, but this is the code point for [A]
        # (Regional Indicator A), minus the code point for ASCII A. By adding
        # this to the uppercase characters making up the ISO 3166-1 alpha-2
        # codes we can get the flag.
        OFFSET = 127397
        points = [ord(x) + OFFSET for x in self.code.upper()]

        try:
            # Python 3 is simple: we can just chr() the unicode points.
            return chr(points[0]) + chr(points[1])
        except ValueError:
            # Python 2 requires us to be a bit more creative. We could use
            # unichr(), but that only works if the python has been compiled
            # with wide unicode support. This method should always work.
            return ('\\U%08x\\U%08x' % tuple(points)).decode('unicode-escape')

    @staticmethod
    def country_from_ioc(ioc_code, flag_url=''):
        code = ioc_data.IOC_TO_ISO.get(ioc_code, '')
        if code == '':
            return None
        return Country(code, flag_url=flag_url)

    @property
    def ioc_code(self):
        return ioc_data.ISO_TO_IOC.get(self.code, '')


class CountryDescriptor(object):
    """
    A descriptor for country fields on a model instance. Returns a Country when
    accessed so you can do things like::

        >>> from people import Person
        >>> person = Person.object.get(name='Chris')

        >>> person.country.name
        'New Zealand'

        >>> person.country.flag
        '/static/flags/nz.gif'
    """
    def __init__(self, field):
        self.field = field

    def __get__(self, instance=None, owner=None):
        if instance is None:
            return self
        # Check in case this field was deferred.
        if self.field.name not in instance.__dict__:
            instance.refresh_from_db(fields=[self.field.name])
        value = instance.__dict__[self.field.name]
        if self.field.multiple:
            return [self.country(code) for code in value]
        return self.country(value)

    def country(self, code):
        return Country(
            code=code, flag_url=self.field.countries_flag_url,
            custom_countries=self.field.countries)

    def __set__(self, instance, value):
        value = self.field.get_clean_value(value)
        instance.__dict__[self.field.name] = value


class LazyChoicesMixin(widgets.LazyChoicesMixin):

    def _set_choices(self, value):
        """
        Also update the widget's choices.
        """
        super(LazyChoicesMixin, self)._set_choices(value)
        self.widget.choices = value


class LazyTypedChoiceField(LazyChoicesMixin, forms.TypedChoiceField):
    """
    A form TypedChoiceField that respects choices being a lazy object.
    """
    widget = widgets.LazySelect


class LazyTypedMultipleChoiceField(
        LazyChoicesMixin, forms.TypedMultipleChoiceField):
    """
    A form TypedMultipleChoiceField that respects choices being a lazy object.
    """
    widget = widgets.LazySelectMultiple


class CountryField(CharField):
    """
    A country field for Django models that provides all ISO 3166-1 countries as
    choices.
    """
    descriptor_class = CountryDescriptor

    def __init__(self, *args, **kwargs):
        countries_class = kwargs.pop('countries', None)
        self.countries = countries_class() if countries_class else countries
        self.countries_flag_url = kwargs.pop('countries_flag_url', None)
        self.blank_label = kwargs.pop('blank_label', None)
        self.multiple = kwargs.pop('multiple', None)
        kwargs['choices'] = self.countries
        if self.multiple:
            kwargs['max_length'] = len(self.countries) * 3 - 1
        else:
            kwargs['max_length'] = 2
        super(CharField, self).__init__(*args, **kwargs)

    def check(self, **kwargs):
        errors = super(CountryField, self).check(**kwargs)
        errors.extend(self._check_multiple())
        return errors

    def _check_multiple(self):
        if not self.multiple or not self.null:
            return []

        hint = 'Remove null=True argument on the field'
        if not self.blank:
            hint += ' (just add blank=True if you want to allow no selection)'
        hint += '.'

        return [
            checks.Error(
                'Field specifies multiple=True, so should not be null.',
                obj=self,
                id='django_countries.E100',
                hint=hint,
            )
        ]

    def get_internal_type(self):
        return "CharField"

    def contribute_to_class(self, cls, name):
        super(CountryField, self).contribute_to_class(cls, name)
        setattr(cls, self.name, self.descriptor_class(self))

    def pre_save(self, *args, **kwargs):
        "Returns field's value just before saving."
        value = super(CharField, self).pre_save(*args, **kwargs)
        return self.get_prep_value(value)

    def get_prep_value(self, value):
        "Returns field's value prepared for saving into a database."
        value = self.get_clean_value(value)
        if self.multiple:
            if value:
                value = ','.join(value)
            else:
                value = ''
        return super(CharField, self).get_prep_value(value)

    def get_clean_value(self, value):
        if value is None:
            return None
        if not self.multiple:
            return country_to_text(value)
        if isinstance(value, (basestring, Country)):
            if isinstance(value, basestring) and ',' in value:
                value = value.split(',')
            else:
                value = [value]
        return list(filter(None, [country_to_text(c) for c in value]))

    def deconstruct(self):
        """
        Remove choices from deconstructed field, as this is the country list
        and not user editable.

        Not including the ``blank_label`` property, as this isn't database
        related.
        """
        name, path, args, kwargs = super(CountryField, self).deconstruct()
        kwargs.pop('choices')
        if self.multiple:      # multiple determines the length of the field
            kwargs['multiple'] = self.multiple
        if self.countries is not countries:
            # Include the countries class if it's not the default countries
            # instance.
            kwargs['countries'] = self.countries.__class__
        return name, path, args, kwargs

    def get_choices(
            self, include_blank=True, blank_choice=None, *args, **kwargs):
        if blank_choice is None:
            if self.blank_label is None:
                blank_choice = BLANK_CHOICE_DASH
            else:
                blank_choice = [('', self.blank_label)]
        if self.multiple:
            include_blank = False
        return super(CountryField, self).get_choices(
            include_blank=include_blank, blank_choice=blank_choice, *args,
            **kwargs)

    get_choices = lazy(get_choices, list)

    def formfield(self, **kwargs):
        argname = 'choices_form_class'
        if argname not in kwargs:
            kwargs[argname] = (
                LazyTypedMultipleChoiceField
                if self.multiple else LazyTypedChoiceField)
        if 'coerce' not in kwargs:
            kwargs['coerce'] = super(CountryField, self).to_python
        field = super(CharField, self).formfield(**kwargs)
        return field

    def to_python(self, value):
        if not self.multiple:
            return super(CountryField, self).to_python(value)
        if not value:
            return value
        output = []
        for item in value:
            output.append(super(CountryField, self).to_python(item))
        return output

    def validate(self, value, model_instance):
        """
        Use custom validation for when using a multiple countries field.
        """
        if not self.multiple:
            return super(CountryField, self).validate(value, model_instance)

        if not self.editable:
            # Skip validation for non-editable fields.
            return

        if value:
            choices = [option_key for option_key, option_value in self.choices]
            for single_value in value:
                if single_value not in choices:
                    raise exceptions.ValidationError(
                        self.error_messages['invalid_choice'],
                        code='invalid_choice',
                        params={'value': single_value},
                    )

        if not self.blank and value in self.empty_values:
            raise exceptions.ValidationError(
                self.error_messages['blank'], code='blank')


FieldListFilter.register(
    lambda f: isinstance(f, CountryField), filters.CountryFilter)



#!/usr/bin/env python
"""
Builds all flags into a single sprite image (along with some css).
"""
import os
import re

from PIL import Image

re_flag_file = re.compile(r'[a-z]{2}.gif$')
FLAG_X, FLAG_Y = 16, 11


def main():
    flag_path = os.path.join(os.path.dirname(__file__), 'static', 'flags')
    files = os.listdir(flag_path)
    img = Image.new('RGBA', (26*FLAG_X, 26*FLAG_Y))
    for name in files:
        if not re_flag_file.match(name):
            continue
        x = (ord(name[0]) - 97) * FLAG_X
        y = (ord(name[1]) - 97) * FLAG_Y
        flag_img = Image.open(os.path.join(flag_path, name))
        img.paste(flag_img, (x, y))
    img.save(os.path.join(flag_path, 'sprite-hq.png'))
    img = img.quantize(method=2, kmeans=1)
    img.save(os.path.join(flag_path, 'sprite.png'))
    css_file = open(os.path.join(flag_path, 'sprite.css'), 'w')
    css_hq_file = open(os.path.join(flag_path, 'sprite-hq.css'), 'w')
    initial_css = (
        '.flag-sprite {display: inline-block;width:%(x)spx;height:%(y)spx;'
        'image-rendering:-moz-crisp-edges;image-rendering:pixelated;'
        'image-rendering:-o-crisp-edges;'
        '-ms-interpolation-mode:nearest-neighbor;'
        "background-image:url('%%s')}" % {'x': FLAG_X, 'y': FLAG_Y})
    css_file.write(initial_css % 'sprite.png')
    write_coords(css_file, FLAG_X, FLAG_Y)
    css_hq_file.write(initial_css % 'sprite-hq.png')
    write_coords(css_hq_file, FLAG_X, FLAG_Y)
    for mult in range(2, 5):
        css_hq_file.write(
            '\n.flag%sx {background-size:%spx %spx}'
            '\n.flag%sx.flag-sprite {width:%spx;height:%spx;}' % (
                mult, 26*FLAG_X*mult, 26*FLAG_Y*mult,
                mult, FLAG_X*mult, FLAG_Y*mult))
        write_coords(
            css_hq_file, FLAG_X*mult, FLAG_Y*mult, prefix='.flag%sx' % mult)


def write_coords(css_file, width, height, prefix=''):
    for i in range(26):
        x, y = i * width, i * height
        if x:
            x = '-{}px'.format(x)
        if y:
            y = '-{}px'.format(y)
        code = chr(i + 97)
        css_file.write(
            '\n%s.flag-%s {background-position-x:%s}' % (prefix, code, x))
        css_file.write(
            '\n%s.flag-_%s {background-position-y:%s}' % (prefix, code, y))


if __name__ == '__main__':
    main()



import django
from django import template
from django_countries.fields import Country, countries


register = template.Library()

if django.VERSION < (1, 9):
    # Support older versions without implicit assignment support in simple_tag.
    simple_tag = register.assignment_tag
else:
    simple_tag = register.simple_tag


@simple_tag
def get_country(code):
    return Country(code=code)


@simple_tag
def get_countries():
    return list(countries)



from django.template import Template, Context
from django.test import TestCase
from django.utils import translation

from django_countries import countries


class TestCountriesTags(TestCase):

    TEMPLATE_COUNTRY = Template(
        "{% load countries %}{% get_country code as country %}{{ country }}")
    TEMPLATE_NAME = Template(
        "{% load countries %}{% get_country code as country %}"
        "{{ country.name }}")
    TEMPLATE_COUNTRIES = Template(
        "{% load countries %}{% get_countries as countries_list %}"
        "{{ countries_list }}")

    def test_country(self):
        rendered = self.TEMPLATE_COUNTRY.render(Context({'code': 'BR'}))
        self.assertEqual(rendered, 'BR')

    def test_country_name(self):
        rendered = self.TEMPLATE_NAME.render(Context({'code': 'BR'}))
        self.assertEqual(rendered, 'Brazil')

    def test_country_name_translated(self):
        with translation.override('pt-BR'):
            rendered = self.TEMPLATE_NAME.render(Context({'code': 'BR'}))
        self.assertEqual(rendered, 'Brasil')

    def test_wrong_code(self):
        rendered = self.TEMPLATE_COUNTRY.render(Context({'code': 'XX'}))
        self.assertEqual(rendered, 'XX')

        rendered = self.TEMPLATE_NAME.render(Context({'code': 'XX'}))
        self.assertEqual(rendered, '')

    def test_countries_displayed_all(self):
        rendered = self.TEMPLATE_COUNTRIES.render(Context())
        self.assertEqual(
            rendered,
            Template("{{ expected }}").render(Context(
                {'expected': [(code, name) for code, name in countries]}))
        )



from __future__ import unicode_literals
from django.test import TestCase
from django.test.client import RequestFactory
from django.contrib import admin
from django.contrib.admin.views.main import ChangeList

from django_countries import filters
from django_countries.tests import models

test_site = admin.AdminSite(name='test-admin')


class PersonAdmin(admin.ModelAdmin):
    list_filter = [
        ('country', filters.CountryFilter),
    ]


test_site.register(models.Person, PersonAdmin)


class TestCountryFilter(TestCase):

    def get_changelist_args(self, **kwargs):
        m = self.person_admin
        args = (
            kwargs.pop('list_display', m.list_display),
            kwargs.pop('list_display_links', m.list_display_links),
            kwargs.pop('list_filter', m.list_filter),
            kwargs.pop('date_hierarchy', m.date_hierarchy),
            kwargs.pop('search_fields', m.search_fields),
            kwargs.pop('list_select_related', m.list_select_related),
            kwargs.pop('list_per_page', m.list_per_page),
            kwargs.pop('list_max_show_all', m.list_max_show_all),
            kwargs.pop('list_editable', m.list_editable),
            m,
        )
        assert not kwargs, "Unexpected kwarg %s" % kwargs
        return args

    def setUp(self):
        models.Person.objects.create(name='Alice', country='NZ')
        models.Person.objects.create(name='Bob', country='AU')
        models.Person.objects.create(name='Chris', country='NZ')
        self.person_admin = PersonAdmin(models.Person, test_site)

    def test_filter_none(self):
        request = RequestFactory().get('/person/')
        cl = ChangeList(request, models.Person, *self.get_changelist_args())
        cl.get_results(request)
        self.assertEqual(
            list(cl.result_list), list(models.Person.objects.all()))

    def test_filter_country(self):
        request = RequestFactory().get('/person/', data={'country': 'NZ'})
        cl = ChangeList(request, models.Person, *self.get_changelist_args())
        cl.get_results(request)
        self.assertEqual(
            list(cl.result_list),
            list(models.Person.objects.exclude(country='AU'))
        )

    def test_choices(self):
        request = RequestFactory().get('/person/', data={'country': 'NZ'})
        cl = ChangeList(request, models.Person, *self.get_changelist_args())
        choices = list(cl.filter_specs[0].choices(cl))
        self.assertEqual(
            [c['display'] for c in choices],
            ['All', 'Australia', 'New Zealand']
        )
        for choice in choices:
            self.assertEqual(
                choice['selected'],
                choice['display'] == 'New Zealand'
            )



from __future__ import unicode_literals
from unittest import skipIf
try:
    from urllib import parse as urlparse
except ImportError:
    import urlparse   # Python 2

from distutils.version import StrictVersion
import django
from django.forms.models import modelform_factory
from django.test import TestCase
from django.utils import safestring
from django.utils.html import escape

from django_countries import widgets, countries, fields
from django_countries.conf import settings
from django_countries.tests.models import Person


class TestCountrySelectWidget(TestCase):

    def setUp(self):
        del countries.countries
        self.Form = modelform_factory(
            Person, fields=['country'],
            widgets={'country': widgets.CountrySelectWidget})

    def tearDown(self):
        del countries.countries

    def test_not_default_widget(self):
        Form = modelform_factory(Person, fields=['country'])
        widget = Form().fields['country'].widget
        self.assertFalse(isinstance(widget, widgets.CountrySelectWidget))

    def test_render_contains_flag_url(self):
        with self.settings(COUNTRIES_ONLY={'AU': 'Desert'}):
            html = self.Form().as_p()
            self.assertIn(escape(urlparse.urljoin(
                settings.STATIC_URL, settings.COUNTRIES_FLAG_URL)), html)

    def test_render(self):
        with self.settings(COUNTRIES_ONLY={'AU': 'Desert'}):
            html = self.Form().as_p()
            self.assertIn(fields.Country('__').flag, html)
            self.assertNotIn(fields.Country('AU').flag, html)

    def test_render_initial(self):
        with self.settings(COUNTRIES_ONLY={'AU': 'Desert'}):
            html = self.Form(initial={'country': 'AU'}).as_p()
            self.assertIn(fields.Country('AU').flag, html)
            self.assertNotIn(fields.Country('__').flag, html)

    def test_render_escaping(self):
        output = widgets.CountrySelectWidget().render('test', '<script>')
        self.assertIn('&lt;script&gt;', output)
        self.assertNotIn('<script>', output)
        self.assertTrue(isinstance(output, safestring.SafeData))

    def test_render_modelform_instance(self):
        person = Person(country='NZ')
        self.Form(instance=person).as_p()

    @skipIf(
        StrictVersion(django.get_version()) < StrictVersion('1.10'),
        'required attribute only implemented in 1.10+')
    def test_required_attribute(self):
        rendered = self.Form()['country'].as_widget()
        rendered = rendered[:rendered.find('>')+1]
        self.assertIn('required', rendered)



from django import forms

from django_countries.tests import models


class PersonForm(forms.ModelForm):

    class Meta:
        model = models.Person
        fields = ['country', 'favourite_country']


class AllowNullForm(forms.ModelForm):

    class Meta:
        model = models.AllowNull
        fields = ['country']


class MultiCountryForm(forms.ModelForm):

    class Meta:
        model = models.MultiCountry
        fields = ['countries']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase
from django.utils import translation

from django_countries import countries, Countries
from django_countries.tests import custom_countries


EXPECTED_COUNTRY_COUNT = 249
FIRST_THREE_COUNTRIES = [
    ('AF', 'Afghanistan'),
    ('AX', 'Åland Islands'),
    ('AL', 'Albania'),
]


class BaseTest(TestCase):

    def setUp(self):
        del countries.countries

    def tearDown(self):
        del countries.countries


class TestCountriesObject(BaseTest):

    def test_countries_len(self):
        self.assertEqual(len(countries), EXPECTED_COUNTRY_COUNT)

    def test_countries_sorted(self):
        self.assertEqual(list(countries)[:3], FIRST_THREE_COUNTRIES)

    def test_countries_limit(self):
        with self.settings(
                COUNTRIES_ONLY={'NZ': 'New Zealand', 'NV': 'Neverland'}):
            self.assertEqual(list(countries), [
                ('NV', 'Neverland'),
                ('NZ', 'New Zealand'),
            ])
            self.assertEqual(len(countries), 2)

    def test_countries_limit_codes(self):
        with self.settings(COUNTRIES_ONLY=['NZ', ('NV', 'Neverland')]):
            self.assertEqual(list(countries), [
                ('NV', 'Neverland'),
                ('NZ', 'New Zealand'),
            ])
            self.assertEqual(len(countries), 2)

    def test_countries_custom_removed_len(self):
        with self.settings(COUNTRIES_OVERRIDE={'AU': None}):
            self.assertEqual(len(countries), EXPECTED_COUNTRY_COUNT - 1)

    def test_countries_custom_added_len(self):
        with self.settings(COUNTRIES_OVERRIDE={'XX': 'Neverland'}):
            self.assertEqual(len(countries), EXPECTED_COUNTRY_COUNT + 1)

    def test_countries_getitem(self):
        countries[0]

    def test_countries_slice(self):
        sliced = countries[10:20:2]
        self.assertEqual(len(sliced), 5)

    def test_countries_custom_ugettext_evaluation(self):

        class FakeLazyUGetText(object):

            def __bool__(self):  # pragma: no cover
                raise ValueError("Can't evaluate lazy_ugettext yet")

            __nonzero__ = __bool__

        with self.settings(COUNTRIES_OVERRIDE={'AU': FakeLazyUGetText()}):
            countries.countries

    def test_ioc_countries(self):
        from ..ioc_data import check_ioc_countries
        check_ioc_countries(verbosity=0)

    def test_initial_iter(self):
        # Use a new instance so nothing is cached
        dict(Countries())

    def test_flags(self):
        from ..data import check_flags
        check_flags(verbosity=0)

    def test_common_names(self):
        from ..data import check_common_names
        check_common_names()

    def test_alpha2(self):
        self.assertEqual(countries.alpha2('NZ'), 'NZ')
        self.assertEqual(countries.alpha2('nZ'), 'NZ')
        self.assertEqual(countries.alpha2('Nzl'), 'NZ')
        self.assertEqual(countries.alpha2(554), 'NZ')
        self.assertEqual(countries.alpha2('554'), 'NZ')

    def test_alpha2_invalid(self):
        self.assertEqual(countries.alpha2('XX'), '')

    def test_alpha2_override(self):
        with self.settings(COUNTRIES_OVERRIDE={'AU': None}):
            self.assertEqual(countries.alpha2('AU'), '')

    def test_alpha2_override_new(self):
        with self.settings(COUNTRIES_OVERRIDE={'XX': 'Neverland'}):
            self.assertEqual(countries.alpha2('XX'), 'XX')

    def test_fetch_by_name(self):
        code = countries.by_name('United States of America')
        self.assertEqual(code, 'US')

    def test_fetch_by_name_case_insensitive(self):
        code = countries.by_name('United states of America')
        self.assertEqual(code, 'US')

    def test_fetch_by_name_old(self):
        code = countries.by_name('Czech Republic')
        self.assertEqual(code, 'CZ')

    def test_fetch_by_name_old_case_insensitive(self):
        code = countries.by_name('Czech republic')
        self.assertEqual(code, 'CZ')

    def test_fetch_by_name_i18n(self):
        code = countries.by_name('Estados Unidos', language='es')
        self.assertEqual(code, 'US')

    def test_fetch_by_name_no_match(self):
        self.assertEqual(countries.by_name('Neverland'), '')


class CountriesFirstTest(BaseTest):

    def test_countries_first(self):
        with self.settings(COUNTRIES_FIRST=['NZ', 'AU']):
            self.assertEqual(
                list(countries)[:5],
                [
                    ('NZ', 'New Zealand'),
                    ('AU', 'Australia'),
                ] + FIRST_THREE_COUNTRIES)

    def test_countries_first_break(self):
        with self.settings(COUNTRIES_FIRST=['NZ', 'AU'],
                           COUNTRIES_FIRST_BREAK='------'):
            self.assertEqual(
                list(countries)[:6],
                [
                    ('NZ', 'New Zealand'),
                    ('AU', 'Australia'),
                    ('', '------'),
                ] + FIRST_THREE_COUNTRIES)

    def test_countries_first_some_valid(self):
        with self.settings(COUNTRIES_FIRST=['XX', 'NZ', 'AU'],
                           COUNTRIES_FIRST_BREAK='------'):
            countries_list = list(countries)
        self.assertEqual(
            countries_list[:6],
            [
                ('NZ', 'New Zealand'),
                ('AU', 'Australia'),
                ('', '------'),
            ] + FIRST_THREE_COUNTRIES)
        self.assertEqual(len(countries_list), EXPECTED_COUNTRY_COUNT + 1)

    def test_countries_first_no_valid(self):
        with self.settings(COUNTRIES_FIRST=['XX'],
                           COUNTRIES_FIRST_BREAK='------'):
            countries_list = list(countries)
        self.assertEqual(countries_list[:3], FIRST_THREE_COUNTRIES)
        self.assertEqual(len(countries_list), EXPECTED_COUNTRY_COUNT)

    def test_countries_first_repeat(self):
        with self.settings(COUNTRIES_FIRST=['NZ', 'AU'],
                           COUNTRIES_FIRST_REPEAT=True):
            countries_list = list(countries)
        self.assertEqual(len(countries_list), EXPECTED_COUNTRY_COUNT + 2)
        sorted_codes = [item[0] for item in countries_list[2:]]
        sorted_codes.index('NZ')
        sorted_codes.index('AU')

    def test_countries_first_len(self):
        with self.settings(COUNTRIES_FIRST=['NZ', 'AU', 'XX']):
            self.assertEqual(len(countries), EXPECTED_COUNTRY_COUNT + 2)

    def test_countries_first_break_len(self):
        with self.settings(COUNTRIES_FIRST=['NZ', 'AU', 'XX'],
                           COUNTRIES_FIRST_BREAK='------'):
            self.assertEqual(len(countries), EXPECTED_COUNTRY_COUNT + 3)

    def test_countries_first_break_len_no_valid(self):
        with self.settings(COUNTRIES_FIRST=['XX'],
                           COUNTRIES_FIRST_BREAK='------'):
            self.assertEqual(len(countries), EXPECTED_COUNTRY_COUNT)

    def test_sorted_countries_first_english(self):
        with self.settings(
                COUNTRIES_FIRST=['GB', 'AF', 'DK'], COUNTRIES_FIRST_SORT=True):
            countries_list = list(countries)
            sorted_codes = [item[0] for item in countries_list[:3]]
            self.assertEqual(['AF', 'DK', 'GB'], sorted_codes)

    def test_unsorted_countries_first_english(self):
        with self.settings(
                COUNTRIES_FIRST=['GB', 'AF', 'DK'],
                COUNTRIES_FIRST_SORT=False):
            countries_list = list(countries)
            unsorted_codes = [item[0] for item in countries_list[:3]]
            self.assertEqual(['GB', 'AF', 'DK'], unsorted_codes)

    def test_sorted_countries_first_arabic(self):
        with self.settings(
                COUNTRIES_FIRST=['GB', 'AF', 'DK'], COUNTRIES_FIRST_SORT=True):
            lang = translation.get_language()
            translation.activate('eo')
            try:
                countries_list = list(countries)
                sorted_codes = [item[0] for item in countries_list[:3]]
                self.assertEqual(['AF', 'GB', 'DK'], sorted_codes)
            finally:
                translation.activate(lang)


class TestCountriesCustom(BaseTest):

    def test_countries_limit(self):
        fantasy_countries = custom_countries.FantasyCountries()
        self.assertEqual(list(fantasy_countries), [
            ('NV', 'Neverland'),
            ('NZ', 'New Zealand'),
        ])
        self.assertEqual(len(fantasy_countries), 2)



from __future__ import unicode_literals

from rest_framework import serializers
from django.utils.encoding import force_text

from django_countries import countries


class CountryField(serializers.ChoiceField):

    def __init__(self, *args, **kwargs):
        self.country_dict = kwargs.pop('country_dict', None)
        field_countries = kwargs.pop('countries', None)
        self.countries = field_countries if field_countries else countries
        super(CountryField, self).__init__(self.countries, *args, **kwargs)

    def to_representation(self, obj):
        code = self.countries.alpha2(obj)
        if not code:
            return ''
        if not self.country_dict:
            return code
        return {'code': code, 'name': force_text(self.countries.name(obj))}

    def to_internal_value(self, data):
        if isinstance(data, dict):
            data = data.get('code')
        country = self.countries.alpha2(data)
        if data and not country:
            country = self.countries.by_name(force_text(data))
            if not country:
                self.fail('invalid_choice', input=data)
        return country



from __future__ import unicode_literals
from django.test import TestCase
from django.utils import six

from django_countries import countries, data, base


class TestSettings(TestCase):

    def setUp(self):
        del countries.countries

    def tearDown(self):
        del countries.countries

    def test_override_additional(self):
        with self.settings(COUNTRIES_OVERRIDE={'XX': 'New'}):
            self.assertEqual(countries.name('XX'), 'New')

    def test_override_replace(self):
        with self.settings(COUNTRIES_OVERRIDE={'NZ': 'Middle Earth'}):
            self.assertEqual(countries.name('NZ'), 'Middle Earth')

    def test_override_remove(self):
        with self.settings(COUNTRIES_OVERRIDE={'AU': None}):
            self.assertNotIn('AU', countries)
            self.assertEqual(countries.name('AU'), '')

    def test_override_only(self):
        with self.settings(COUNTRIES_ONLY={'AU': 'Desert'}):
            self.assertTrue(len(countries.countries) == 1)
            self.assertIn('AU', countries)
            self.assertEqual(countries.name('AU'), 'Desert')

    def test_common_names(self):
        common_code, common_name = (
            list(base.CountriesBase.COMMON_NAMES.items())[0])
        common_name = six.text_type(common_name)
        name = six.text_type(countries.countries[common_code])
        self.assertEqual(name, common_name)

        del countries.countries
        official_name = six.text_type(data.COUNTRIES[common_code])
        with self.settings(COUNTRIES_COMMON_NAMES=False):
            name = six.text_type(countries.countries[common_code])
            self.assertEqual(name, official_name)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import pickle

from django.db import models
from django.core import validators, checks
from django.forms import Select
from django.forms.models import modelform_factory
from django.test import TestCase, override_settings
from django.utils import translation
from django.utils.encoding import force_text

from django_countries import fields, countries, data
from django_countries.fields import CountryField
from django_countries.tests import forms, custom_countries
from django_countries.tests.models import (
    Person, AllowNull, MultiCountry, WithProp)


class TestCountryField(TestCase):

    def test_logic(self):
        person = Person(name='Chris Beaven', country='NZ')

        self.assertEqual(person.country, 'NZ')
        self.assertNotEqual(person.country, 'ZZ')

        self.assertTrue(person.country)
        person.country = ''
        self.assertFalse(person.country)

    def test_get_property_from_class(self):
        self.assertIsInstance(Person.country, fields.CountryDescriptor)

    def test_deconstruct(self):
        field = Person._meta.get_field('country')
        self.assertEqual(
            field.deconstruct(),
            ('country', 'django_countries.fields.CountryField', [],
             {'max_length': 2}))

    def test_text(self):
        person = Person(name='Chris Beaven', country='NZ')
        self.assertEqual(force_text(person.country), 'NZ')

    def test_name(self):
        person = Person(name='Chris Beaven', country='NZ')
        self.assertEqual(person.country.name, 'New Zealand')

    def test_flag(self):
        person = Person(name='Chris Beaven', country='NZ')
        with self.settings(STATIC_URL='/static-assets/'):
            self.assertEqual(
                person.country.flag, '/static-assets/flags/nz.gif')

    def test_custom_field_flag_url(self):
        person = Person(name='Chris Beaven', country='NZ', other_country='US')
        self.assertEqual(
            person.other_country.flag, '//flags.example.com/us.gif')

    def test_unicode_flags(self):
        person = Person(
            name='Matthew Schinckel', country='AU', other_country='DE')
        self.assertEqual(person.country.unicode_flag, '🇦🇺')
        self.assertEqual(person.other_country.unicode_flag, '🇩🇪')

    def test_unicode_flag_blank(self):
        person = Person(name='Matthew Schinckel')
        self.assertEqual(person.country.unicode_flag, '')

    def test_COUNTRIES_FLAG_URL_setting(self):
        # Custom relative url
        person = Person(name='Chris Beaven', country='NZ')
        with self.settings(COUNTRIES_FLAG_URL='img/flag-{code_upper}.png',
                           STATIC_URL='/static-assets/'):
            self.assertEqual(
                person.country.flag, '/static-assets/img/flag-NZ.png')
        # Custom absolute url
        with self.settings(COUNTRIES_FLAG_URL='https://flags.example.com/'
                           '{code_upper}.PNG'):
            self.assertEqual(
                person.country.flag, 'https://flags.example.com/NZ.PNG')

    def test_flag_css(self):
        person = Person(name='Chris Beaven', country='NZ')
        self.assertEqual(person.country.flag_css, 'flag-sprite flag-n flag-_z')

    def test_flag_css_blank(self):
        person = Person(name='Chris Beaven')
        self.assertEqual(person.country.flag_css, '')

    def test_blank(self):
        person = Person.objects.create(name='The Outsider')
        self.assertEqual(person.country.code, '')

        person = Person.objects.get(pk=person.pk)
        self.assertEqual(person.country.code, '')

    def test_null(self):
        person = AllowNull.objects.create(country=None)
        self.assertIsNone(person.country.code)

        person = AllowNull.objects.get(pk=person.pk)
        self.assertIsNone(person.country.code)

    @override_settings(SILENCED_SYSTEM_CHECKS=['django_countries.E100'])
    def test_multi_null_country(self):

        class MultiNullCountry(models.Model):
            countries = fields.CountryField(
                multiple=True, null=True, blank=True)

        class MultiNullCountryNoBlank(models.Model):
            countries = fields.CountryField(
                multiple=True, null=True)

        errors = checks.run_checks()
        self.assertEqual([e.id for e in errors], ['django_countries.E100'] * 2)
        errors_dict = dict((e.obj, e) for e in errors)
        self.assertFalse(
            'blank=True' in errors_dict[
                MultiNullCountry._meta.get_field('countries')
            ].hint)
        self.assertTrue(
            'blank=True' in errors_dict[
                MultiNullCountryNoBlank._meta.get_field('countries')
            ].hint)

    def test_deferred(self):
        Person.objects.create(name='Person',
                              country='NZ')
        person = Person.objects.defer('country').get(name='Person')
        self.assertEqual(person.country.code, 'NZ')

    def test_only(self):
        Person.objects.create(name='Person',
                              country='NZ')
        person = Person.objects.only('name').get()
        self.assertEqual(person.country.code, 'NZ')

    def test_nullable_deferred(self):
        AllowNull.objects.create(country=None)
        person = AllowNull.objects.defer('country').get()
        self.assertIsNone(person.country.code)

    def test_len(self):
        person = Person(name='Chris Beaven', country='NZ')
        self.assertEqual(len(person.country), 2)

        person = Person(name='The Outsider', country=None)
        self.assertEqual(len(person.country), 0)

    def test_lookup_text(self):
        Person.objects.create(name='Chris Beaven', country='NZ')
        Person.objects.create(name='Pavlova', country='NZ')
        Person.objects.create(name='Killer everything', country='AU')

        lookup = Person.objects.filter(country='NZ')
        names = lookup.order_by('name').values_list('name', flat=True)
        self.assertEqual(list(names), ['Chris Beaven', 'Pavlova'])

    def test_lookup_country(self):
        Person.objects.create(name='Chris Beaven', country='NZ')
        Person.objects.create(name='Pavlova', country='NZ')
        Person.objects.create(name='Killer everything', country='AU')

        oz = fields.Country(code='AU', flag_url='')
        lookup = Person.objects.filter(country=oz)
        names = lookup.values_list('name', flat=True)
        self.assertEqual(list(names), ['Killer everything'])

    def test_save_empty_country(self):
        Person.objects.create(name='The Outsider')
        person = Person.objects.get()
        self.assertEqual(person.country.code, '')

    def test_create_modelform(self):
        Form = modelform_factory(Person, fields=['country'])
        form_field = Form().fields['country']
        self.assertTrue(isinstance(form_field.widget, Select))

    def test_render_form(self):
        Form = modelform_factory(Person, fields=['country'])
        Form().as_p()

    def test_model_with_prop(self):
        with_prop = WithProp(country='FR', public_field='test')

        self.assertEqual(with_prop.country.code, 'FR')
        self.assertEqual(with_prop.public_field, 'test')

    def test_in(self):
        Person.objects.create(name='A', country='NZ')
        Person.objects.create(name='B', country='AU')
        Person.objects.create(name='C', country='FR')
        Person.objects.create(name='D', country='NZ')

        self.assertEqual(
            list(
                Person.objects.filter(country__in=['AU', 'NZ'])
                .values_list('name', flat=True)),
            ['A', 'B', 'D']
        )


class TestValidation(TestCase):

    def test_validate(self):
        person = Person(name='Chris', country='NZ')
        person.full_clean()

    def test_validate_alpha3(self):
        person = Person(name='Chris', country='NZL')
        person.full_clean()

    def test_validate_empty(self):
        person = Person(name='Chris')
        self.assertRaises(validators.ValidationError, person.full_clean)

    def test_validate_invalid(self):
        person = Person(name='Chris', country=':(')
        self.assertRaises(validators.ValidationError, person.full_clean)

    def test_validate_multiple(self):
        person = MultiCountry(countries=['NZ', 'AU'])
        person.full_clean()

    def test_validate_multiple_empty(self):
        person = MultiCountry()
        self.assertRaises(validators.ValidationError, person.full_clean)

    def test_validate_multiple_invalid(self):
        person = MultiCountry(countries=[':(', 'AU'])
        self.assertRaises(validators.ValidationError, person.full_clean)

    def test_validate_multiple_uneditable(self):
        person = MultiCountry(countries='NZ', uneditable_countries='xx')
        person.full_clean()

    def test_get_prep_value_empty_string(self):
        country_field_instance = CountryField(multiple=True, blank=True)
        prep_value = country_field_instance.get_prep_value('')
        self.assertEqual(prep_value, '')

    def test_get_prep_value_none(self):
        """
        Note: django migrations will call get_prep_value() with None
        see: https://github.com/SmileyChris/django-countries/issues/215
        """
        country_field_instance = CountryField(multiple=True, blank=True)
        prep_value = country_field_instance.get_prep_value(None)
        self.assertEqual(prep_value, '')


class TestCountryCustom(TestCase):

    def test_name(self):
        person = Person(name='Chris Beaven', fantasy_country='NV')
        self.assertEqual(person.fantasy_country.name, 'Neverland')

    def test_field(self):
        self.assertEqual(
            list(Person._meta.get_field('fantasy_country').choices),
            [('NV', 'Neverland'), ('NZ', 'New Zealand')])

    def test_deconstruct(self):
        field = Person._meta.get_field('fantasy_country')
        self.assertEqual(
            field.deconstruct(),
            (
                'fantasy_country',
                'django_countries.fields.CountryField',
                [],
                {
                    'countries': custom_countries.FantasyCountries,
                    'blank': True,
                    'max_length': 2
                }
            ))


class TestCountryMultiple(TestCase):

    def test_empty(self):
        obj = MultiCountry()
        self.assertEqual(obj.countries, [])

    def test_empty_save(self):
        MultiCountry.objects.create()

    def test_single(self):
        obj = MultiCountry(countries='NZ')
        self.assertEqual(len(obj.countries), 1)
        self.assertTrue(isinstance(obj.countries[0], fields.Country))
        self.assertEqual(obj.countries[0], 'NZ')

    def test_multiple(self):
        obj = MultiCountry(countries='AU,NZ')
        self.assertEqual(len(obj.countries), 2)
        for country in obj.countries:
            self.assertTrue(isinstance(country, fields.Country))
        self.assertEqual(obj.countries[0], 'AU')
        self.assertEqual(obj.countries[1], 'NZ')

    def test_set_text(self):
        obj = MultiCountry()
        obj.countries = 'NZ,AU'
        self.assertEqual(obj.countries, ['NZ', 'AU'])

    def test_set_list(self):
        obj = MultiCountry()
        obj.countries = ['NZ', 'AU']
        self.assertEqual(obj.countries, ['NZ', 'AU'])

    def test_set_country(self):
        obj = MultiCountry()
        obj.countries = fields.Country('NZ')
        self.assertEqual(obj.countries, ['NZ'])

    def test_set_countries(self):
        obj = MultiCountry()
        obj.countries = [fields.Country('NZ'), fields.Country('AU')]
        self.assertEqual(obj.countries, ['NZ', 'AU'])

    def test_all_countries(self):
        all_codes = list(c[0] for c in countries)
        MultiCountry.objects.create(countries=all_codes)
        obj = MultiCountry.objects.get()
        self.assertEqual(obj.countries, all_codes)

    def test_deconstruct(self):
        field = MultiCountry._meta.get_field('countries')
        expected_max_length = len(data.COUNTRIES) * 3 - 1
        self.assertEqual(
            field.deconstruct(),
            (
                'countries',
                'django_countries.fields.CountryField',
                [],
                {'max_length': expected_max_length, 'multiple': True}
            ))


class TestCountryObject(TestCase):

    def test_hash(self):
        country = fields.Country(code='XX', flag_url='')
        self.assertEqual(hash(country), hash('XX'))

    def test_repr(self):
        country1 = fields.Country(code='XX')
        country2 = fields.Country(code='XX', flag_url='')
        country3 = fields.Country(code='XX', str_attr='name')
        self.assertEqual(
            repr(country1),
            'Country(code={0})'.format(repr('XX')))
        self.assertEqual(
            repr(country2),
            'Country(code={0}, flag_url={1})'.format(repr('XX'), repr('')))
        self.assertEqual(
            repr(country3),
            'Country(code={0}, str_attr={1})'.format(repr('XX'), repr('name')))

    def test_str(self):
        country = fields.Country(code='NZ')
        self.assertEqual('%s' % country, 'NZ')

    def test_str_attr(self):
        country = fields.Country(code='NZ', str_attr='name')
        self.assertEqual('%s' % country, 'New Zealand')

    def test_flag_on_empty_code(self):
        country = fields.Country(code='', flag_url='')
        self.assertEqual(country.flag, '')

    def test_ioc_code(self):
        country = fields.Country(code='NL', flag_url='')
        self.assertEqual(country.ioc_code, 'NED')

    def test_country_from_ioc_code(self):
        country = fields.Country.country_from_ioc('NED')
        self.assertEqual(country, fields.Country('NL', flag_url=''))

    def test_country_from_blank_ioc_code(self):
        country = fields.Country.country_from_ioc('')
        self.assertIsNone(country)

    def test_country_from_nonexistence_ioc_code(self):
        country = fields.Country.country_from_ioc('XXX')
        self.assertIsNone(country)

    def test_alpha3(self):
        country = fields.Country(code='BN')
        self.assertEqual(country.alpha3, 'BRN')

    def test_alpha3_invalid(self):
        country = fields.Country(code='XX')
        self.assertEqual(country.alpha3, '')

    def test_numeric(self):
        country = fields.Country(code='BN')
        self.assertEqual(country.numeric, 96)

    def test_numeric_padded(self):
        country = fields.Country(code='AL')
        self.assertEqual(country.numeric_padded, '008')
        country = fields.Country(code='BN')
        self.assertEqual(country.numeric_padded, '096')
        country = fields.Country(code='NZ')
        self.assertEqual(country.numeric_padded, '554')

    def test_numeric_invalid(self):
        country = fields.Country(code='XX')
        self.assertEqual(country.numeric, None)

    def test_numeric_padded_invalid(self):
        country = fields.Country(code='XX')
        self.assertEqual(country.numeric_padded, None)

    def test_empty_flag_url(self):
        country = fields.Country(code='XX', flag_url='')
        self.assertEqual(country.flag, '')

    def test_alpha2_code(self):
        country = fields.Country(code='NZL')
        self.assertEqual(country.code, 'NZ')

    def test_alpha2_code_invalid(self):
        country = fields.Country(code='NZX')
        self.assertEqual(country.code, 'NZX')

    def test_numeric_code(self):
        country = fields.Country(code=554)
        self.assertEqual(country.code, 'NZ')

    def test_numeric_code_invalid(self):
        country = fields.Country(code=999)
        self.assertEqual(country.code, 999)


class TestModelForm(TestCase):

    def test_translated_choices(self):
        lang = translation.get_language()
        translation.activate('eo')
        form = forms.PersonForm()
        try:
            # This is just to prove that the language changed.
            self.assertEqual(list(countries)[0][1], 'Afganio')
            # If the choices aren't lazy, this wouldn't be translated. It's the
            # second choice because the first one is the initial blank option.
            self.assertEqual(
                form.fields['country'].choices[1][1], 'Afganio')
            self.assertEqual(
                form.fields['country'].widget.choices[1][1], 'Afganio')
        finally:
            translation.activate(lang)

    def test_blank_choice(self):
        blank = ('', '---------')

        form = forms.PersonForm()
        self.assertEqual(form.fields['country'].choices[0], blank)

        multi_form = forms.MultiCountryForm()
        self.assertNotEqual(
            multi_form.fields['countries'].choices[0], blank)

    def test_no_blank_choice(self):
        form = forms.PersonForm()
        self.assertEqual(
            form.fields['favourite_country'].choices[0], ('AF', 'Afghanistan'))

    def test_blank_choice_label(self):
        form = forms.AllowNullForm()
        self.assertEqual(
            form.fields['country'].choices[0], ('', '(select country)'))

    def test_validation(self):
        form = forms.MultiCountryForm(data={'countries': ['NZ', 'AU']})
        self.assertEqual(form.errors, {})


class TestPickling(TestCase):

    def test_standard_country_pickling(self):
        chris = Person(name='Chris Beaven', country='NZ')
        # django uses pickle.HIGHEST_PROTOCOL which is somewhere between 2 and
        # 4, depending on python version. Let's use 2 for testing.
        newly_pickled_zealand = pickle.dumps(chris.country, protocol=2)
        # Different python versions end up with slightly different sizes. Let's
        # just check the size is smaller than if it contained the entire
        # standard countries list in the pickle.
        self.assertLess(len(newly_pickled_zealand), 200)

        unpickled = pickle.loads(newly_pickled_zealand)
        self.assertEqual(unpickled.code, 'NZ')
        self.assertEqual(unpickled.name, 'New Zealand')
        self.assertEqual(unpickled.flag_url, None)
        self.assertIs(unpickled.countries, countries)
        self.assertIsNone(unpickled.custom_countries)

    def test_custom_country_pickling(self):
        chris = Person(name='Chris Beaven', fantasy_country='NV')
        # django uses pickle.HIGHEST_PROTOCOL which is somewhere between 2 and
        # 4, depending on python version. Let's use 2 for testing.
        pickled_neverland = pickle.dumps(chris.fantasy_country, protocol=2)
        # Different python versions end up with slightly different sizes. Let's
        # just check the size is smaller than if it also contained the fantasy
        # countries list in the pickle.
        self.assertLess(len(pickled_neverland), 300)

        neverland = pickle.loads(pickled_neverland)
        self.assertEqual(neverland.code, 'NV')
        self.assertEqual(neverland.name, 'Neverland')
        self.assertEqual(neverland.flag_url, None)
        self.assertIsInstance(
            neverland.countries, custom_countries.FantasyCountries)



from __future__ import unicode_literals

from django.test import TestCase, override_settings

from django_countries import countries
from django_countries.fields import Country
from django_countries.tests.models import Person, MultiCountry
from django_countries.tests.custom_countries import FantasyCountries
from django_countries.serializers import CountryFieldMixin

from rest_framework.test import APIRequestFactory
from rest_framework import serializers, views


def countries_display(countries):
    """
    Convert Countries into a DRF-OPTIONS formatted dict.
    """
    return [{'display_name': v, 'value': k} for (k, v) in countries]


class PersonSerializer(CountryFieldMixin, serializers.ModelSerializer):

    class Meta:
        model = Person
        fields = (
            'name', 'country', 'other_country', 'favourite_country',
            'fantasy_country')
        extra_kwargs = {
            'other_country': {'country_dict': True},
        }


class MultiCountrySerializer(CountryFieldMixin, serializers.ModelSerializer):

    class Meta:
        model = MultiCountry
        fields = ('countries',)


class TestDRF(TestCase):

    def test_serialize(self):
        person = Person(name='Chris Beaven', country='NZ')
        serializer = PersonSerializer(person)
        self.assertEqual(
            serializer.data,
            {
                'name': 'Chris Beaven',
                'country': 'NZ',
                'other_country': '',
                'favourite_country': 'NZ',
                'fantasy_country': '',
            })

    def test_serialize_country_dict(self):
        person = Person(name='Chris Beaven', other_country='AU')
        serializer = PersonSerializer(person)
        self.assertEqual(
            serializer.data,
            {
                'name': 'Chris Beaven',
                'country': '',
                'other_country': {'code': 'AU', 'name': 'Australia'},
                'favourite_country': 'NZ',
                'fantasy_country': '',
            })

    def test_deserialize(self):
        serializer = PersonSerializer(data={
            'name': 'Tester',
            'country': 'US',
            'fantasy_country': 'NV',
        })
        self.assertTrue(serializer.is_valid())
        self.assertEqual(serializer.validated_data['country'], 'US')

    def test_deserialize_country_dict(self):
        serializer = PersonSerializer(data={
            'name': 'Tester',
            'country': {'code': 'GB', 'name': 'Anything'},
            'fantasy_country': 'NV'
        })
        self.assertTrue(serializer.is_valid())
        self.assertEqual(serializer.validated_data['country'], 'GB')

    def test_deserialize_by_name(self):
        serializer = PersonSerializer(data={
            'name': 'Chris',
            'country': 'New Zealand',
            'fantasy_country': 'Neverland',
        })
        self.assertTrue(serializer.is_valid())
        self.assertEqual(serializer.validated_data['country'], 'NZ')
        self.assertEqual(serializer.validated_data['fantasy_country'], 'NV')

    def test_deserialize_invalid(self):
        serializer = PersonSerializer(data={
            'name': 'Chris',
            'country': 'No Such Zealand',
            'fantasy_country': 'Neverland',
        })
        self.assertFalse(serializer.is_valid())
        self.assertTrue(
            serializer.errors['country'],
            ['"No Such Zealand" is not a valid choice.'])

    def test_multi_serialize(self):
        mc = MultiCountry(countries='NZ,AU')
        serializer = MultiCountrySerializer(mc)
        self.assertEqual(
            serializer.data,
            {
                'countries': ['NZ', 'AU']
            })

    def test_multi_serialize_empty(self):
        mc = MultiCountry(countries='')
        serializer = MultiCountrySerializer(mc)
        self.assertEqual(
            serializer.data,
            {
                'countries': []
            })

    def test_multi_deserialize(self):
        serializer = MultiCountrySerializer(data={
            'countries': ['NZ', 'AU']
        })
        self.assertTrue(serializer.is_valid())
        self.assertEqual(serializer.validated_data['countries'], ['NZ', 'AU'])

    def test_multi_deserialize_by_name(self):
        serializer = MultiCountrySerializer(data={
            'countries': ['New Zealand', 'Australia']
        })
        self.assertTrue(serializer.is_valid())
        self.assertEqual(serializer.validated_data['countries'], ['NZ', 'AU'])

    def test_multi_deserialize_invalid(self):
        serializer = MultiCountrySerializer(data={
            'countries': ['NZ', 'BAD', 'AU']
        })
        self.assertFalse(serializer.is_valid())
        self.assertEqual(
            serializer.errors['countries'],
            ['"BAD" is not a valid choice.'])

    def test_multi_deserialize_save(self):
        serializer = MultiCountrySerializer(data={
            'countries': ['NZ', 'AU']
        })
        self.assertTrue(serializer.is_valid())
        saved = serializer.save()
        loaded = MultiCountry.objects.get(pk=saved.pk)
        self.assertEqual(loaded.countries, [Country('NZ'), Country('AU')])


class TestDRFMetadata(TestCase):
    """
    Tests against the DRF OPTIONS API metadata endpoint.
    """

    def test_actions(self):

        class ExampleView(views.APIView):
            """Example view."""
            def post(self, request):
                pass  # pragma: nocover

            def get_serializer(self):
                return PersonSerializer()

        def _choices(response, key):
            """Helper method for unpacking response JSON."""
            return response.data['actions']['POST'][key]['choices']

        view = ExampleView.as_view()

        factory = APIRequestFactory()
        request = factory.options('/')
        response = view(request=request)
        country_choices = _choices(response, 'country')
        fantasy_choices_en = _choices(response, 'fantasy_country')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(country_choices, countries_display(countries))
        self.assertEqual(
            fantasy_choices_en, countries_display(FantasyCountries()))

        with override_settings(LANGUAGE_CODE='fr'):
            response = view(request=request)
            fantasy_choices_fr = _choices(response, 'fantasy_country')
            self.assertNotEqual(fantasy_choices_en, fantasy_choices_fr)
